# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import re

from functools import partial
from subprocess import PIPE

from law.util import colored

from analysis.tasks.FnloCppread import FnloCppread

from analysis.framework import Task
from analysis.util import interruptable_popen, working_directory

logger = logging.getLogger(__name__)
working_directory = partial(working_directory, logger=logger)

class Combine(Task):
    merge_dir = luigi.Parameter(default='grids')
    tags_dir = luigi.Parameter(default='tags')

    cores = luigi.Parameter()
    combine_ini = luigi.Parameter()

    def requires(self):
        return {'tables': FnloCppread()}

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalFileTarget(
            '{}/{}/{}/{}.{}.combine.log'.format(
                out_dir,
                self.tags_dir, self.name, self.process, self.name)
        )

    def run(self):
        # backported `enum34` is incompatible with Python > 3.4
        # -> remove it from PYTHONPATH
        env = os.environ.copy()
        env["PYTHONPATH"] = os.pathsep.join(p for p in env["PYTHONPATH"].split(os.pathsep) if not re.search('enum34', p))

        # ensure output parent directory exists
        logger.debug("Creating output parent directory: %s", self.output().parent.path)
        self.output().parent.touch()

        outfile = self.output().path
        errfile = re.sub('.log$', '.err', outfile)

        rundirpath = os.getcwd()
        combine_ini = os.path.abspath(self.combine_ini)

        mergedir = '{}/{}'.format(self.merge_dir, self.name)

        with working_directory(mergedir):
            command_line_args = [
                'nnlojet-combine.py', '-C', combine_ini, '-j', self.cores
            ]
            logger.info("Running command: %s",
                colored(' '.join(command_line_args), 'yellow'))
            code, out, error = interruptable_popen(command_line_args,
                                                   stdout=PIPE,
                                                   stderr=PIPE,
                                                   env=env)

            logger.debug("Return code: %d", code)
            logger.debug("Standard output:\n%s\n", out)
            logger.debug("Standard error:\n%s\n", error)

        if error:
            logger.debug("Dumping standard error to file:  %s", errfile)
            with open(errfile, 'w') as f:
                f.write(error)

        if (code == 0):
            logger.info("nnlojet-combine status: SUCCESS")
            logger.debug("Dumping standard out to file:  %s", outfile)
            self.output().dump(out, formatter="text")
        else:
            error_message = "nnlojet-combine status: FAIL (exit code {})".format(code)
            logger.error(error_message)
            raise RuntimeError(error_message)
